import uuid
from datetime import datetime

from django.db.models import (
    Model,
    BooleanField,
    CharField,
    TextField,
    DateTimeField,
    ForeignKey,
    UUIDField,
    CASCADE,
)
from django.utils.translation import gettext_lazy as _
from django.conf import settings
from autoslug.fields import AutoSlugField

from .managers import BasePublicationManager
from .statuses import Statuses


class BasePublication(Model):
    title = CharField(
        max_length=512,
        verbose_name=_('Title'),
    )
    slug = AutoSlugField(
        db_index=True,
        blank=True,
        unique=True,
        verbose_name=_('Slug'),
        populate_from='title',
    )
    description = TextField(
        verbose_name=_('Description'),
        blank=True,
        null=True,
    )
    status = CharField(
        max_length=255,
        choices=Statuses,
        default=Statuses.DRAFT.value,
        db_index=True,
        verbose_name=_('Status'),
    )
    owner = ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=CASCADE,
        blank=True,
        verbose_name=_('Owner'),
    )
    publication_date = DateTimeField(
        default=datetime.now,
        db_index=True,
        verbose_name=_('Publication date'),
    )
    created_date = DateTimeField(
        auto_now_add=True,
        editable=False,
        verbose_name=_('Created date'),
    )
    last_modified_date = DateTimeField(
        auto_now=True,
        editable=False,
        verbose_name=_('Last modified date'),
    )
    is_hidden = BooleanField(
        default=False,
        db_index=True,
        verbose_name=_('Is hidden'),
    )
    token = UUIDField(
        default=uuid.uuid4,
        blank=True,
        editable=False,
        verbose_name=_('Token')
    )

    objects = BasePublicationManager()

    @property
    def is_published(self):
        return self.status == Statuses.PUBLISHED.value

    @property
    def canonical_url(self):
        return self.get_absolute_url()

    def get_absolute_url(self):
        raise NotImplementedError('get_absolute_url not implemented')

    class Meta:
        abstract = True
        ordering = [
            '-publication_date',
        ]




from collections import Iterable
from random import shuffle
from datetime import datetime

from django.db.models import Manager
from .statuses import Statuses


class BasePublicationManager(Manager):
    def get_published(self):
        return self.filter(
            is_hidden=False,
            status=Statuses.PUBLISHED.value,
            publication_date__lte=datetime.now(),
        )

    def get_random(self, exclude_ids=None, category_slug=None, limit=3):
        query = self.get_published()

        if exclude_ids and isinstance(exclude_ids, Iterable):
            query = query.exclude(id=exclude_ids) if isinstance(exclude_ids, str) else query.exclude(id__in=exclude_ids)

        id_list = [item.id for item in query]
        shuffle(id_list)

        return self.filter(id__in=id_list[:limit])



from urllib.parse import urlunsplit
from django.contrib.admin import ModelAdmin
from django.utils.translation import gettext_lazy as _


class PublicationAdmin(ModelAdmin):
    list_display = (
        'title',
        'status',
        'publication_date',
        'owner',
    )
    readonly_fields = (
        'created_date',
        'last_modified_date',
        'guest_link',
    )

    list_filter = [
        'status',
        'owner',
        'is_hidden',
        'publication_date',
        'created_date',
        'last_modified_date',
    ]
    search_fields = (
        'title',
        'slug',
    )

    fieldsets = (
        (None, {
            'fields': (
                'title',
                'status',
                'publication_date',
                'is_hidden',
            ),
        }),
        (_('Additionally'), {
            'fields': (
                'owner',
                'slug',
                'guest_link',
            ),
        })
    )

    def guest_link(self, obj):
        if obj.pk:
            return urlunsplit((obj.absolute_url, f'token={obj.token}', ''))
        return ""

    guest_link.short_description = _('Slug')

    def save_model(self, request, obj, form, change):
        if not getattr(obj, 'owner', None):
            obj.owner = request.user
            
        super().save_model(request, obj, form, change)



from ..utils import NamedEnum


class Statuses(NamedEnum):
    DRAFT = 'Draft'
    EDITING_REQUIRED = 'Editing required'
    READY_TO_PUBLISH = 'Ready to publish'
    PROOFREADING_REQUIRED = 'Proofreading required'
    PUBLISHED = 'Published'



from enum import Enum, EnumMeta
from django.utils.translation import gettext_lazy as _


class NamedEnumMeta(EnumMeta):
    def __iter__(cls):
        return (
            (cls._member_map_[name].value, _(cls._member_map_[name].value))
            for name in cls._member_names_
        )


class NamedEnum(Enum, metaclass=NamedEnumMeta):
    pass



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['django_base_publications',
 'django_base_publications.migrations',
 'django_base_publications.models']

package_data = \
{'': ['*'], 'django_base_publications': ['locale/ru/LC_MESSAGES/*']}

install_requires = \
['django-autoslug>=1.9,<2.0',
 'django>=1.11,<3.0',
 'funcy>=1.13,<2.0',
 'ipython>=7.7,<8.0']

setup_kwargs = {
    'name': 'django-base-publications',
    'version': '0.0.6',
    'description': '',
    'long_description': None,
    'author': 'Дмитрий',
    'author_email': 'acrius@mail.ru',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



from datetime import datetime, timedelta
from django.contrib.sitemaps import Sitemap


class BasePublicationSitemap(Sitemap):
    model = None

    def items(self):
        return self.model.objects.get_published()

    def lastmod(self, obj):
        return obj.publication_date


class BaseGoogleNewsSitemap(Sitemap):
    protocol = 'https'
    limit = 1000
    model = None

    def items(self):
        date = datetime.now() - timedelta(days=2)
        return self.model.objects.get_published().filter(publication_date__gte=date)



from django.contrib.syndication.views import Feed
from django.template.defaultfilters import striptags
from django.contrib.sites.shortcuts import get_current_site
from django.contrib.syndication.views import add_domain
from django.utils.feedgenerator import Rss201rev2Feed
from django.utils.html import escape
from django.utils.http import urlquote
from funcy import last

from .settings import LOGO_URL, SQUARE_LOGO_URL


class CustomFeedGenerator(Rss201rev2Feed):
    content_type = 'text/xml; charset=utf-8'

    def __init__(self, title, link, description, **kwargs):
        super().__init__(title, link, description, **kwargs)
        # For removing atom link from xml document
        self.feed['feed_url'] = None

    def rss_attributes(self):
        return {
            'xmlns:yandex': 'https://news.yandex.ru',
            'xmlns:media': 'https://search.yahoo.com/mrss/',
            'version': self._version,
        }

    def add_root_elements(self, handler):
        super().add_root_elements(handler)
        logo = self.feed.get('logo')
        logo_square = self.feed.get('logo_square')

        if logo:
            handler.addQuickElement('yandex:logo', logo)

        if logo_square:
            handler.addQuickElement(
                'yandex:logo',
                logo_square,
                {'type': 'square'},
            )

    def add_item_elements(self, handler, item):
        super(CustomFeedGenerator, self).add_item_elements(handler, item)
        handler.addQuickElement('yandex:full-text', item['content'])

        images = item.get('images', [])

        for image in images:
            handler.addQuickElement('enclosure', '', {
                'url': image.get('url'),
                'type': image.get('type'),
            })


class ZenGenerator(CustomFeedGenerator):
    def rss_attributes(self):
        return {
            'xmlns:yandex': 'http://news.yandex.ru',
            'xmlns:content': 'http://purl.org/rss/1.0/modules/content/',
            'version': self._version,
        }

    def add_item_elements(self, handler, item):
        super(ZenGenerator, self).add_item_elements(handler, item)
        handler.addQuickElement("content:encoded", "<![CDATA[%s]]>" % striptags(item["content"]))


class MailFeedGenerator(CustomFeedGenerator):
    def rss_attributes(self):
        return {
            'xmlns:content': 'http://purl.org/rss/1.0/modules/content/',
            'version': self._version,
        }

    def add_item_elements(self, handler, item):
        super(MailFeedGenerator, self).add_item_elements(handler, item)
        handler.addQuickElement("amplink", '{}amp/'.format(item["link"]))


class CustomFeed(Feed):
    feed_type = CustomFeedGenerator
    model = None
    items_per_feed = 20

    def items(self):
        return self.model.objects.get_published()[:self.items_per_feed]

    def feed_extra_kwargs(self, obj):
        kwargs = super().feed_extra_kwargs(obj)

        if LOGO_URL:
            kwargs['logo'] = self.get_with_domain(LOGO_URL)

        if SQUARE_LOGO_URL:
            kwargs['logo_square'] = self.get_with_domain(SQUARE_LOGO_URL)

        return kwargs

    def item_title(self, item):
        return escape(item.title)

    def item_description(self, item):
        return escape(striptags(item.description)) if item.description \
            else escape(striptags(item.content))

    def item_pubdate(self, item):
        return item.publication_date

    def item_author_name(self, item):
        return item.owner.get_full_name() if item.owner else None

    def item_author_email(self, item):
        return item.owner.email if item.owner else None

    def item_extra_kwargs(self, item):
        kwargs = super().item_extra_kwargs(item)

        # add images for <enclosure> tags
        images = self.item_images(item)

        kwargs.update({
            'content': escape(striptags(self.item_content(item))),
            'images': self._prepare_item_images(images),
        })
        return kwargs

    def _prepare_item_images(self, images):
        return [self._prepare_item_image(image) for image in images]

    def _prepare_item_image(self, image):
        extensions = last(image.name.split('.')).lower()
        image_url = urlquote(image.url).replace('%3A', ':')

        return {
            'image': image,
            'url': self.get_with_domain(image_url),
            'type': 'image/{}'.format(extensions.replace('jpg', 'jpeg')),
        }

    def item_images(self, item):
        """
        Return item images fields.
        """
        return []

    def item_content(self, item):
        return getattr(item, 'content', '')

    def get_with_domain(self, link):
        site = get_current_site(self.request)
        return add_domain(site.domain, link, self.request.is_secure)

    def __call__(self, request, *args, **kwargs):
        self.request = request
        return super().__call__(request, *args, **kwargs)
